/*
 * Copyright (c)1997 by Dianne Hackborn.
 * All rights reserved.
 *
 * Based (increasingly loosly) on WebTerm, a Web-based terminal
 * applet written in Java, which is
 * Copyright (C)1996 by the National Alliance for Computational
 * Science and Engineering (NACSE).
 * See <URL:http://www.nacse.org/> for more information.
 *
 * This code is not public domain, nor freely distributable.
 * Please direct any questions or requests to Dianne Hackborn,
 * at <hackbod@lucent.com> or <hackbod@enteract.com>.
 *
 * ----------------------------------------------------------------------
 *
 * PTYRemote.h
 *
 * This class converts the ArpRemoteTerminal BMessage-based interface
 * into read/write file handles.
 *
 * ----------------------------------------------------------------------
 *
 * Known Bugs
 * ~~~~~~~~~~
 *
 *	• There are surely lots and lots of bugs that aren't
 *	  known...  but soon will be.
 *
 * ----------------------------------------------------------------------
 *
 * History
 * ~~~~~~~
 *
 * 7/28/1998:
 *	•	Created from the ArpTelnet source.
 *
 */

#ifndef PTYREMOTE_H
#define PTYREMOTE_H

#ifndef _LOCKER_H
#include <be/support/Locker.h>
#endif

#ifndef _LOOPER_H
#include <be/app/Looper.h>
#endif

#ifndef _MESSENGER_H
#include <be/app/Messenger.h>
#endif

#ifndef __BSTRING__
#include <be/support/String.h>
#endif

#ifndef ARPKERNEL_ARPSTRING_H
#include <ArpKernel/ArpString.h>
#endif

#ifndef ARPKERNEL_ARPCONFIGUREWATCH_H
#include <ArpKernel/ArpConfigureWatch.h>
#endif

#ifndef ARPTERMINAL_ARPREMOTEADDON_H
#include <ArpTerminal/ArpRemoteAddon.h>
#endif

#ifndef ARPTERMINAL_ARPRTERMINALMSG_H
#include <ArpTerminal/ArpTerminalMsg.h>
#endif

class ArpPTYRemote : public BLooper, public ArpRemoteInterface {
private:
	typedef BLooper inherited;

public:

	static const ArpRemoteAddon AddonInterface;
	
	/* ------------------------------------------------------------
	   CONTRUCTORS AND GLOBAL CONTROL METHODS
	   ------------------------------------------------------------ */

	ArpPTYRemote(const BMessenger& term, const char* url);
	ArpPTYRemote(const char* url = NULL);
	
	virtual void Delete() {
		Lock();
		Quit();
	}
	
	virtual status_t GetConfiguration(BMessage* values) const;
	virtual status_t PutConfiguration(const BMessage* values);
	virtual status_t Configure(ArpVectorI<BView*>& views);
	
	virtual const ArpRemoteType* RemoteType(void) const;
	
	virtual BMessenger GetDevice(void) { return BMessenger(this); }
	
	// You don't actually need to call these, as the terminal
	// will handshake with its remote device when the device
	// is attached to it.
	void SetTerminal(const BMessenger& term) { terminal = term; }
	BMessenger GetTerminal(void) { return terminal; }
	
	// Note: these functions return immediately; the object will
	// still be in the process of connecting or disconnecting when
	// control returns to the caller.
	status_t Connect(BWindow* owner = NULL);
	status_t Connect(const char* url, BWindow* owner = NULL);
	void Disconnect();

	virtual bool IsConnected() const	{ return connected; }
	virtual bool IsInUse() const		{ return mInUse; }
	
	/* ------------------------------------------------------------
	   DATA IN CONFIGURATION MESSAGES.
	   ------------------------------------------------------------ */

	static const char* CommandConfigName;
	
	/* ------------------------------------------------------------
	   CONSTANT VALUES SECTION
	   ------------------------------------------------------------ */
	
protected:

	// NOTE: None of the protected methods lock the BLooper;
	// you must do so yourself, if needed.
	
	/* ------------------------------------------------------------
	   MESSAGE HANDLING
	   ------------------------------------------------------------ */

	void initialize(const char* url);
	
	void MessageReceived(BMessage* message);
	
	// Send received text to terminal
	void receive(const char* str, int32 len=-1);
	
	void changeEmulator(const char* name);
	
	// Prompts aren't implemented.
	void clearPrompts();

	status_t DoConnect(const char* url = NULL);
	void DoDisconnect();
	
	virtual ~ArpPTYRemote();
	
	/* ------------------------------------------------------------
	   DATA SENDING METHODS
	   ------------------------------------------------------------ */

	void setWindowSize(int32 width, int32 height, bool force=false);
	void setEmulator(const char* name);
	void write(char c);
	void write(const char* b, int32 len);
	void write(const ArpString& str);
	
	/* ------------------------------------------------------------
	   INPUT THREAD METHODS
	   ------------------------------------------------------------ */

	/* Entry point and main loop.
	   Note well: we must be careful that any methods that read
	   from the stream (either directly with in.read(), or indirectly
	   with next_char()) do not have the looper locked.  Otherwise,
	   they may block and leave other threads unable to access the
	   object and write more data to the socket. */

	static int32 readThreadEntry(void* arg);
	int32 startReader(void);
	int32 runReader(void);
	
	/* Input instance data.  This may only be accessed by the input
	   thread. */

	/* ------------------------------------------------------------
	   MISC SUPPORT METHODS
	   ------------------------------------------------------------ */

	/* Parse escapes in a parameter string. */
	ArpString parseString(const ArpString& in);

	/* ------------------------------------------------------------
	   INSTANCE DATA SECTION
	   ------------------------------------------------------------ */

	BMessenger terminal;
	
	/* Common instance data, used by all threads.  Methods that
	   access it must be protected by locking this locker.
	   NOTE: To avoid deadlocks, you -must- -not- lock the
	   BLooper after locking the 'globals' semaphore.
	*/

	BLocker globals;
	
	ArpConfigureWatch mWatch;		// Standard configurable object implementation
	
	BString cmdline;				// Command line to start shell
	int ptyfd;						// Communication with shell
	
	bool doEcho;					// Echo typed characters back to terminal?
	bool waitingReply;				// Waiting for reply to last text sent to term?
	ArpString writeBuf;				// Buffer that is waiting for reply.
	
	thread_id readThread;			// Thread for reading socket.
	status_t socketError;			// Last socket read error.
	thread_id shellThread;			// Thread that is running shell.
	bool connected;					// TRUE if reader is active.
	bool mInUse;					// TRUE if user has done something.
	bool mDisconnectRequested;		// TRUE if disconnect was requested by user.
	
	BMessage emuTypes;				// Available emulation types
	int32 curEmuType;
	ArpString curEmuName;
	
	int32 curWidth, curHeight;		// Current window dimensions.
};

#endif
