/*
 * ArpTelnet Copyright (c)1997-98 by Dianne Hackborn.
 * All rights reserved.
 *
 * Based (increasingly loosly) on WebTerm, a Web-based terminal
 * applet written in Java, which is
 * Copyright (C)1996 by the National Alliance for Computational
 * Science and Engineering (NACSE).
 * See <URL:http://www.nacse.org/> for more information.
 *
 * This code is not public domain, nor freely distributable.
 * Please direct any questions or requests to Dianne Hackborn,
 * at <hackbod@lucent.com> or <hackbod@angryredplanet.com>.
 *
 * ----------------------------------------------------------------------
 *
 * TerminalWin.h
 *
 * The TerminalWin class defines a complete window with a
 * terminal placed inside it.  It attaches a VT200 emulator
 * to the terminal, creates an ArpTelnet device, and connects
 * the telnet device with the terminal so that the user can
 * interact with its telnet sessions.
 *
 * ----------------------------------------------------------------------
 *
 * Known Bugs
 * ~~~~~~~~~~
 *
 * ----------------------------------------------------------------------
 *
 * History
 * ~~~~~~~
 *
 * 0.1: Created this file.
 *
 */

#ifndef TERMINALWIN_H
#define TERMINALWIN_H

#ifndef _WINDOW_H
#include <be/interface/Window.h>
#endif

#ifndef ARPTERMINAL_ARPEMULATORMANAGER_H
#include <ArpTerminal/ArpEmulatorManager.h>
#endif

#ifndef ARPTERMINAL_ARPREMOTEMANAGER_H
#include <ArpTerminal/ArpRemoteManager.h>
#endif

#ifndef ARPKERNEL_ARPCONFIGUREFILE_H
#include <ArpKernel/ArpConfigureFile.h>
#endif

#ifndef ARPKERNEL_ARPCONFIGUREWATCH_H
#include <ArpKernel/ArpConfigureWatch.h>
#endif

enum {
	// These both have two values:
	// "current" (entry_ref) the current settings file.
	// "who" (BMessenger) the window asking to save/open.
	TERMINALWIN_SAVESETTINGS_MSG = '.SST',
	TERMINALWIN_OPENSETTINGS_MSG = '.OST'
};

class ArpRemoteTerminal;

class TerminalWin : public BWindow, public ArpConfigurableI {
private:
	typedef BWindow inherited;
	
public:
	TerminalWin(BPoint pos, const char* title,
				const entry_ref& settings = entry_ref(),
				const char* abouttxt = "About" B_UTF8_ELLIPSIS);
	~TerminalWin();
	
	status_t OpenConfiguration(const entry_ref& ref, bool connect = true);
	status_t SaveConfiguration(entry_ref dir = entry_ref(),
							   const char* name = 0,
							   bool showError = true,
							   bool moveTo = true);
							   
	ArpRemoteTerminal* Terminal(void)	{ return terminal; }
	
	status_t SetRemote(const char* url, bool connect=true);
	void ClearRemote();
	ArpRemoteInterface* Remote() const	{ return remote; }
	
	void PrintToTerm(ArpString str, int32 flags=0);
	void ResizeToTerm(int32 rows=-1, int32 cols=-1);
	bool HasPosition() const { return mHasPosition; }
	
	virtual void DispatchMessage(BMessage *message,
								 BHandler *handler);
	virtual void MessageReceived(BMessage *message);
	virtual	bool QuitRequested();
	virtual void MenusBeginning(void);
	virtual void Zoom(BPoint leftTop, float width, float height);
	
	/* ------------------------------------------------------------
	   DATA IN CONFIGURATION MESSAGES.
	   ------------------------------------------------------------ */

	static const char* TerminalConfigName;			// BMessage
	static const char* RemoteSettingsConfigName;	// BMessage
	static const char* RemoteProtocolConfigName;	// String
	static const char* LocationConfigName;			// BPoint
	static const char* SizeConfigName;				// BPoint (width, height)
	static const char* WorkspaceConfigName;			// int32
	static const char* CloseConfirmConfigName;		// bool
	static const char* ExitOnCloseConfigName;		// bool
	static const char* SaveOnExitConfigName;		// int32
	
	// These are not actually saved in the settings file -- they control
	// whether some of the above settings are saved.
	static const char* StartAtLocationConfigName;	// int32
	static const char* StartAtSizeConfigName;		// int32
	static const char* StartInWorkspaceConfigName;	// int32
	
	// The ArpConfigurableI interface.  We use this to aggregate all
	// of our configurable objects [ArpTerminal and ArpRemoteInterface],
	// and add in global terminal settings.
	virtual status_t GetConfiguration(BMessage* values) const;
	virtual status_t PutConfiguration(const BMessage* values);
	virtual status_t Configure(ArpVectorI<BView*>& views);

	// Return the configuration values for only this object -- not
	// any other aggregated objects such as ArpTerminal.
	virtual status_t GetThisConfiguration(BMessage* values) const;
	
protected:
	void UpdateTitle();
	status_t GetManualRef(entry_ref* ref);
	
private:
	ArpConfigureFile mConfigFile;
	ArpConfigureFile mDefConfig;
	ArpEmulatorManager mEmulators;
	ArpRemoteManager mRemotes;
	BString mDefTitle;
	BString mTermTitle;
	
	ArpConfigureWatch mWatch;
	
	BMenu* sessionmenu;
	BMenu* connmenu;
	
	BMenuItem* connitem;
	BMenuItem* disconnitem;
	
	BMenu* winsizemenu;
	BMenuItem* lockitem;
	
	BMenu* fontsizemenu;
	BMenu* encodingmenu;
	
	BMenuItem* crlfitem;
	BMenuItem* enteritem;
	BMenuItem* bsdelitem;
	BMenuItem* wrapitem;
	BMenuItem* inverseitem;
	BMenuItem* pqueryitem;
	BMenuItem* rmbpasteitem;
	BMenuItem* iscrollitem;
	BMenuItem* oscrollitem;
	
	BMenu* emumenu;
	
	ArpRemoteTerminal* terminal;
	ArpRemoteInterface* remote;
	bool mDoConnect;
	bool mHasPosition;
	bool mStartAtLocation;
	bool mStartAtSize;
	bool mStartInWorkspace;
	bool mCloseConfirm;
	bool mExitOnClose;
	int32 mSaveOnExit;
	
	// If IsValid(), the currently open settings window.
	BMessenger mSettings;
	
	void update_modes(void);
	bool CheckRemoteMenu();
};

#endif
